/* Topology Builder        Bruce D. Ray
   IUPUI Physics Dept. NMR Center
   402 N. Blackford St.
   Indianapolis, IN   46202
   USA
        Functions to adjust general amber forcefield atom type assignments

   NOTICE: This is a derivative work based on study of routines found in
   the antechamber 1.27 program atomtype, version 1.0, dated October, 2001
   by Junmei Wang, Department of Pharmaceutical Chemistry, School of Pharmacy,
   University of California, San Francisco, CA  94143
*/

#include <stdio.h>
#include <math.h>
#include <ctype.h>
#include <stdlib.h>
#include <string.h>
#include "block_memory.h"
#include "adjustments.h"
#include "atom_types.h"
#include "mol2.h"
#include "rings.h"

#define MAXCHAR 256

/* adjust gaff types cc, ce, cg, pc, pe, nc, and ne to be
   gaff types cd, cf, ch, pd, pf, nd, or nf with some bonding
   situations
   parameters are:
        num_atoms                    number of atoms
        num_bonds                    number of bonds
        assigned_type                the types as assigned by type judging
*/
void adjustments(int num_atoms, int num_bonds, char **assigned_type)
{
     int i, indx, num, flag, bonda, bondb;
     int *indxa, *indxb;

     i = num_atoms + 10;
     indxa = (int *)allocator(i, sizeof(int), FARGS);
     indxb = (int *)allocator(i, sizeof(int), FARGS);

     for(i = 0; i < num_atoms; i++) {
          indxa[i] = 0;
          indxb[i] = 0;
     }

     indx = 0;
     num = 0;

     for(i = 0; i < num_atoms; i++)
          if(!strcmp(assigned_type[i], "cc") ||
             !strcmp(assigned_type[i], "ce") ||
             !strcmp(assigned_type[i], "cg") ||
             !strcmp(assigned_type[i], "pc") ||
             !strcmp(assigned_type[i], "pe") ||
             !strcmp(assigned_type[i], "nc") ||
             !strcmp(assigned_type[i], "ne")) {
                  indxb[i] =1;
                  if(!indx) {
                       indxa[i] = 1;
                       indx = 1;
                  }
                  num++;
          }

     if(!num) {
          free(indxa);
          free(indxb);
          return;
     }

     num--;
     while(num > 0) {
          num--;
          flag = 0;
          for(i = 0; i < num_bonds; i++) {
               bonda = bond_i[i];
               bondb = bond_j[i];
               if((indxb[bonda] + indxb[bondb]) != 2) continue;
               if(!flag && !indxa[bonda] && !indxa[bondb])
                    indxa[bonda] = 1;
               if(!indxa[bonda] && indxa[bondb]) {
                    flag = 1;
                    if((how_bonded[i] == 1) ||
                       (how_bonded[i] == 7)) indxa[bonda] = indxa[bondb];
                    if((how_bonded[i] == 2) ||
                       (how_bonded[i] == 8) ||
                       (how_bonded[i] == 3)) indxa[bonda] = -indxa[bondb];
               }                    /* end if(!indxa[bonda] && indxa[bondb]) */

               if(!indxa[bondb] && indxa[bonda]) {
                    flag = 1;
                    if((how_bonded[i] == 1) ||
                       (how_bonded[i] == 7))
                            indxa[bondb] = indxa[bonda];
                    if((how_bonded[i] == 2) ||
                       (how_bonded[i] == 8) ||
                       (how_bonded[i] == 3))
                            indxa[bondb] = -indxa[bonda];
               }                    /* end if(!indxa[bondb] && indxa[bonda]) */
          }                         /* end for(i = 0; i < num_bonds; i++) */
     }                              /* end while(num > 0) */

     for(i = 0; i < num_atoms; i++)
          if(indxa[i] == -1) {
               if(!strcmp(assigned_type[i], "cc"))
                    strcpy(assigned_type[i], "cd");
               if(!strcmp(assigned_type[i], "ce"))
                    strcpy(assigned_type[i], "cf");
               if(!strcmp(assigned_type[i], "cg"))
                    strcpy(assigned_type[i], "ch");
               if(!strcmp(assigned_type[i], "pc"))
                    strcpy(assigned_type[i], "pd");
               if(!strcmp(assigned_type[i], "pe"))
                    strcpy(assigned_type[i], "pf");
               if(!strcmp(assigned_type[i], "nc"))
                    strcpy(assigned_type[i], "nd");
               if(!strcmp(assigned_type[i], "ne"))
                    strcpy(assigned_type[i], "nf");
          }                          /* end if(indxa[i] == -1) */

     free(indxa);
     free(indxb);

     return;
}

/* adjust gaff type cp to be cq in some bonding situations
   parameters are:
        num_atoms                    number of atoms
        num_bonds                    number of bonds
        assigned_type                the types as assigned by type judging
*/
void adjust_type_cp(int num_atoms, int num_bonds, char **assigned_type)
{
     int i, indx, num, flag, bonda, bondb;
     int *indxa, *indxb;

     i = num_atoms + 10;
     indxa = (int *)allocator(i, sizeof(int), FARGS);
     indxb = (int *)allocator(i, sizeof(int), FARGS);

     for(i = 0; i < num_atoms; i++) {
          indxa[i] = 0;
          indxb[i] = 0;
     }

     indx = 0;
     num = 0;

     for(i = 0; i < num_atoms; i++)
          if(!strcmp(assigned_type[i], "cp")) {
               indxb[i] = 1;
               if(!indx) {
                    indxa[i] = 1;
                    indx =1;
               }
               num++;
          }

     if(!num) {
          free(indxa);
          free(indxb);
          return;
     }

     num--;

     while(num > 0) {
          num--;
          for(i = 0; i < num_bonds; i++) {
               bonda = bond_i[i];
               bondb = bond_j[i];
               if((indxb[bonda] + indxb[bondb]) != 2) continue;
               if(!indxa[bonda] && indxa[bondb]) {
                    if(how_bonded[i] == 1)
                         indxa[bonda] = indxa[bondb];
                    else
                         indxa[bonda] = -indxa[bondb];
               }                    /* end if(!indxa[bonda] && indxa[bondb]) */

               if(!indxa[bondb] && indxa[bonda]) {
                    if(how_bonded[i] == 1)
                         indxa[bonda] = indxa[bondb];
                    else
                         indxa[bonda] = -indxa[bondb];
               }                    /* end if(!indxa[bondb] && indxa[bonda]) */
          }                         /* end for(i = 0; i < num_bonds; i++) */
     }                              /* end while(num > 0) */

     for(i = 0; i < num_atoms; i++)
          if((indxa[i] == -1) && !strcmp(assigned_type[i], "cp"))
               strcpy(assigned_type[i], "cq");

     free(indxa);
     free(indxb);

     return;
}

/* check for gaff typing errors for types cc, ce, cg, pc, pe, nc, ne,
   cd, cf, ch, pd, pf, nd, nf, cp, and cq
   parameters are:
        num_atoms                    number of atoms
        num_bonds                    number of bonds
        assigned_type                the types as assigned by type judging
*/
void check_type_errors(int num_atoms, int num_bonds, char **assigned_type)
{
     int *indxr;
     int bondi, bondj, i;

     i = num_atoms + 10;
     indxr = (int *)allocator(i, sizeof(int), FARGS);

     for(i = 0; i < num_atoms; i++) {
          indxr[i] = 0;
          if(!strcmp(assigned_type[i], "cc") ||
             !strcmp(assigned_type[i], "ce") ||
             !strcmp(assigned_type[i], "nc") ||
             !strcmp(assigned_type[i], "ne") ||
             !strcmp(assigned_type[i], "pc") ||
             !strcmp(assigned_type[i], "pe") ||
             !strcmp(assigned_type[i], "cg"))  indxr[i] = 1;

          if(!strcmp(assigned_type[i], "cd") ||
             !strcmp(assigned_type[i], "cf") ||
             !strcmp(assigned_type[i], "nd") ||
             !strcmp(assigned_type[i], "nf") ||
             !strcmp(assigned_type[i], "pd") ||
             !strcmp(assigned_type[i], "pf") ||
             !strcmp(assigned_type[i], "ch"))  indxr[i] = -1;
     }

     for(i = 0; i < num_bonds; i++) {
          bondi = bond_i[i];
          bondj = bond_j[i];
          if((indxr[bondi] == 1) && (indxr[bondj] == 1) ||
             (indxr[bondi] == -1) && (indxr[bondj] == -1))
                  if((how_bonded[i] != 1) && (how_bonded[i] != 7))
                       warn_me(bondi, bondj, assigned_type);

          if(indxr[bondi]*indxr[bondj] == -1)
               if((how_bonded[i] == 1) || (how_bonded[i] == 7))
                    warn_me(bondi, bondj, assigned_type);

          if((!strcmp(assigned_type[bondi], "cp") && !strcmp(assigned_type[bondj], "cp")) ||
             (!strcmp(assigned_type[bondi], "cq") && !strcmp(assigned_type[bondj], "cq")))
                  if(how_bonded[i] != 1)
                       warn_me(bondi, bondj, assigned_type);

          if((!strcmp(assigned_type[bondi], "cp") && !strcmp(assigned_type[bondj], "cq")) ||
             (!strcmp(assigned_type[bondi], "cq") && !strcmp(assigned_type[bondj], "cp")))
                  if(how_bonded[i] == 1)
                       warn_me(bondi, bondj, assigned_type);
     }

     free(indxr);

     return;
}

/* print warning of type errors
   parameters are:
        id1                       id of first atom in bad pair
        id2                       id of second atom in bad pair
        assigned_type                the types as assigned by type judging
*/
void warn_me(int id1, int id2, char **assigned_type)
{
     printf("\nWARNING: atom type of %5s (%s) and %5s (%s) may be wrong\n",
            atom_name[id1], assigned_type[id1], atom_name[id2], assigned_type[id2]);

     return;
}
