/* Topology Builder        Bruce D. Ray
   IUPUI Physics Dept. NMR Center
   402 N. Blackford St.
   Indianapolis, IN   46202
   USA
        Function to locate impropers         improper.c

   NOTICE: This is a derivative work based on study of the routines
   found in the antechamber 1.27 program parmchk, version 1.0,
   dated October, 2001 by Junmei Wang, Department of Pharmaceutical
   Chemistry, School of Pharmacy, University of California,
   San Francisco, CA  94143

   Portions of this work simplify storage allocation, and clarify
   decision trees compared to the work cited above.
*/

#include <stdio.h>
#include <math.h>
#include <ctype.h>
#include <stdlib.h>
#include <string.h>
#include "block_memory.h"
#include "mainchain.h"
#include "mol2.h"
#include "rings.h"

#define MAXCHAR 256
#define MAXNAME 128

/* locate impropers
   parameter is:
        atom_num          number of atoms
*/
void loc_improper(int atom_num)
{
     int i, j, indx, tmpint;
     char mess[MAXCHAR];

     improper_num = 0;

     for(i = 0; i < atom_num; i++) {
          improper[i] = 0;
          if((atom_atno[i] == 6) && (bond_count[i] == 3))
               improper[i] = 1;

          if((atom_atno[i] == 7) && (bond_count[i] == 3) &&
             ((ar_set[i][1] >= 1) || (ar_set[i][2] >= 1) ||
              (ar_set[i][3] >= 1))) improper[i] = 1;

          if(!(strcmp(&atom_type[i][0], "C.2"))) {		       
               indx = 0;
               for(j = 0; j < 3; j++) {
                    if((tmpint = connect[i][j]) == -1) break;
                    if(atom_atno[tmpint] == 7) {
                         indx = 1;
                         break;
                    }
               }
               if(indx) improper[i] = 1;
          }

          if((atom_atno[i] == 7) && !(strcmp(&atom_type[i][0], "N.pl3")))
               improper[i] = 1;

          if((atom_atno[i] == 7) && !(ar_set[i][1]) && !(ar_set[i][2]) &&
              !(ar_set[i][3]) && (bond_count[i] == 3)) {
                    indx = 0;
                    for(j = 0; j < 3; j++) {
                         if((tmpint = connect[i][j]) == -1) break;
                         if((ar_set[tmpint][1] >= 1) || (ar_set[tmpint][2] >= 1) ||
                            (ar_set[tmpint][3] >= 1)) {
                                   indx = 1;
                                   break;
                         }
                    }
                    if(indx) improper[i] = 1;
          }


          if(improper[i]) {
               if((connect[i][0] < 0) || (connect[i][1] < 0) ||
                  (connect[i][2] < 0)) continue;
               improper_num++;
          }
     }

/* make a table of atom numbers in each improper candidate */

     if(improper_num) {                              /* skip if no candidates */
          improperid = (int **)allocator(improper_num, sizeof(int*), FARGS);

          for(i = 0; i < improper_num; i++)
               improperid[i] = (int *)allocator(4, sizeof(int), FARGS);

          j = 0;
          for(i = 0; i < atom_num; i++) {               
               if(improper[i]) {
                    if((connect[i][0] < 0) || (connect[i][1] < 0) ||
                       (connect[i][2] < 0)) continue;                  /* should never take this branch */
                    improperid[j][2] = i;
                    improperid[j][0] = connect[i][0];
                    improperid[j][1] = connect[i][1];
                    improperid[j][3] = connect[i][2];
                    j++;
               }
               if(j > improper_num ) {
                    sprintf(mess, "Improper locus counts mismatch %d > %d\n",
                             j, improper_num);
                    my_fatal(FARGS, mess);
                    exit(1);
               }
          }
     }

     return;
}
