/* Topology Builder        Bruce D. Ray
   IUPUI Physics Dept. NMR Center
   402 N. Blackford St.
   Indianapolis, IN   46202
   USA
        Define storage for molecule from mol2 file and for the ancillary derived
        atom types, forcefield, and measurements data
*/

#define MAXNAME 128

/* information directly from .mol2 file */
                    /* header information */

char mol_name[MAXNAME];          /* molecule name record */
char model_type[MAXNAME];        /* mol2 Molecule model type record */
char charge_type[MAXNAME];       /* mol2 Molecule charge method */

                    /* end header information */

                    /* atoms information */

char **atom_name;                /* list of atom names in mol2 atoms record */
char **atom_type;                /* list of atom types in mol2 atoms record */
char **atom_residue;             /* residue in which atom appears in mol2 atoms record */
int *atom_segno;                 /* residue or chain id in mol2 atoms record */
float *atom_charge;              /* atom charge from mol2 atoms record */

                    /* end atoms information */

                    /* coordinates */

float *atom_x;                   /* atom x coordinate from mol2 atoms record */
float *atom_y;                   /* atom y coordinate from mol2 atoms record */
float *atom_z;                   /* atom z coordinate from mol2 atoms record */

                    /* end coordinates */

                    /* bond information */

char **bond_type;                /* type of bond from mol2 bonds record */
int *bond_i;                     /* first atom in bond from mol2 bonds record */
int *bond_j;                     /* second atom in bond from mol2 bonds record */
int *bond_count;                 /* number of atoms bound to this atom based on mol2 bonds record */
int **connect;                   /* table of atom connections based on mol2 bonds record */
int **bonds_list;                /* table of which bonds each atom participates in */

                    /* end bond information */

                    /* restraints information */

int rstr_dist;                   /* number of simple distance restraints */
int rstr_range;                  /* number of distance range restraints */
int rstr_angle;                  /* number of angle restraints */
int rstr_torsn;                  /* number of torsion restraints */
int **restr;                     /* type and atoms [i][0] is type, & [i][1 to 4] are atom numbers */
float **frest;                   /* restraint values and force constants */

                    /* end restraints information */
/* end information directly from .mol2 file */

/* derived molecule information */

int *atom_sat;
int *atom_ewd;
float *atom_mass;                /* atom mass from table search */
int *atom_atno;                  /* atomic number from table search */
int *how_bonded;                 /* translation of bond type */
float *meas_len;                 /* measured bond length based on coordinates */
char **atom_symbl;               /* element symbol in proper case */
char **a_ff_type;                /* generalized amber force field type */
int **angle_tab;                 /* table of atom numbers in an angle */
float *meas_ang;                 /* measured angles based on coordinates */
int **torsion_tab;               /* table of atom numbers in a torsion or improper torsion */
float *meas_tors;                /* measured torsions based on coordinates */

                     /* improper locus information */

int improper_num;                /* number marked improper by simple search */
int *improper;                   /* flag for marked improper atoms */
int **improperid;                /* atoms connected to each marked improper */
float *meas_impro;               /* measured improper torsion based on coordinates */

                    /* end improper locus information */

                    /* residue atom ordering information */

int *atom_order;                  /* ordering of the atoms for output.  Solves
                                     problem of all residue atoms required to be
                                     together in gromacs topologies */
int *back_order;                  /* reverse of ordering of the atoms for output. */
int true_atoms;                   /* true count of atoms */

                    /* end residue atom ordering information */
/* end derived molecule information */

/* information from force field files */
               /* read from selected amber force field */

float *a_pol;                   /* atom polarizability from selected amber */

float *a_bond_force;            /* bond force from selecter amber */
float *a_bond_length;           /* bond length from selecter amber */

float *a_angle_force;           /* angle force from selected amber */
float *a_angle_angle;           /* angle length from selected amber */
int *a_meth;                    /* angle assignment method */

int *a_tors_mult;               /* torsion multiplier factor from selected amber */
float *a_tors_phase;            /* torsion phase from selected amber */
float *a_tors_force;            /* torsion force from selected amber */
float *a_tors_term;             /* the torsion term from selected amber */

int *a_impr_mult;               /* improper multiplier factor from selected amber */
float *a_impr_force;            /* improper force from selected amber */
float *a_impr_phase;            /* improper phase from selected amber */
float *a_impr_term;             /* the improper term from selected amber */

float *a_vdw_atom_radius;       /* van der Waals radius from selected amber */
float *a_vdw_atom_pot;          /* van der Waals potential from selected amber */

               /* end read from selected amber force field */
/* end information from force field files */
