/* Topology Builder        Bruce D. Ray
   IUPUI Physics Dept. NMR Center
   402 N. Blackford St.
   Indianapolis, IN   46202
   USA
        Function to allocate remainder of needed memory for molecule's
        parameters            molec_alloc.c
*/
#include <stdio.h>
#include <math.h>
#include <ctype.h>
#include <stdlib.h>
#include <string.h>
#include "block_memory.h"
#include "mol2.h"
#include "readmol2.h"

/* allocate force field storage for the molecule
   parameters are:
        num_atoms          number of atoms in the molecule
        num_bonds          number of bonds in the molecule
        num_angles         number of angles between bonds in the molecule
        num_tors           number of torsions around bonds in the molecule
        num_impr           number of improper torsions in the molecule
*/
void molec_alloc(int num_atoms, int num_bonds, int num_angles, int num_tors,
                 int num_impr)
{
     int i;

     a_pol = (float *)allocator(num_atoms, sizeof(float), FARGS);
     a_bond_force = (float *)allocator(num_bonds, sizeof(float), FARGS);
     a_bond_length = (float *)allocator(num_bonds, sizeof(float), FARGS);
     a_angle_force = (float *)allocator(num_angles, sizeof(float), FARGS);
     a_angle_angle = (float *)allocator(num_angles, sizeof(float), FARGS);
     a_meth = (int *)allocator(num_angles, sizeof(int), FARGS);
     a_tors_mult = (int *)allocator(num_tors, sizeof(int), FARGS);
     a_tors_force = (float *)allocator(num_tors, sizeof(float), FARGS);
     a_tors_phase = (float *)allocator(num_tors, sizeof(float), FARGS);
     a_tors_term = (float *)allocator(num_tors, sizeof(float), FARGS);
     a_impr_mult = (int *)allocator(num_impr, sizeof(int), FARGS);
     a_impr_force = (float *)allocator(num_impr, sizeof(float), FARGS);
     a_impr_phase = (float *)allocator(num_impr, sizeof(float), FARGS);
     a_impr_term = (float *)allocator(num_impr, sizeof(float), FARGS);
     a_vdw_atom_radius = (float *)allocator(num_atoms, sizeof(float), FARGS);
     a_vdw_atom_pot = (float *)allocator(num_atoms, sizeof(float), FARGS);

/* set everything to mark unused (don't use 0 here except on method flags) */

     for(i = 0; i < num_atoms; i++){
          a_pol[i] = 99999.0;
          a_vdw_atom_radius[i] = 99999.0;
          a_vdw_atom_pot[i] =  99999.0;
     }

     for(i = 0; i < num_bonds; i++){
          a_bond_force[i] = 99999.0;
          a_bond_length[i] = 99999.0;
     }

     for(i = 0; i < num_angles; i++){
          a_angle_force[i] = 99999.0;
          a_angle_angle[i] = 99999.0;
          a_meth[i] = 0;
     }

     for(i = 0; i < num_tors; i++){
          a_tors_mult[i] = 9999;
          a_tors_force[i] = 99999.0;
          a_tors_phase[i] = 99999.0;
          a_tors_term[i] = 99999.0;
     }

     for(i = 0; i < num_impr; i++){
          a_impr_mult[i] = 9999;
          a_impr_force[i] = 99999.0;
          a_impr_phase[i] = 99999.0;
          a_impr_term[i] = 99999.0;
     }
     return;
}

/* set order of atoms according to residue.  
   Solves problem of all residue atoms required to be
   together in gromacs topologies and of zero mass things.
   Much easier to set up a forward and reverse table look-up
   system than to try sort atoms, bonds, and connection tables.
   parameters are:
        num_atoms                number of atoms in molecule
        resid                    residue name if set by -r option.
                                 NULL otherwise.
*/
void atom_residue_order(int num_atoms, char *resid)
{
     int i, j, k, count;

     atom_order = (int *)allocator(num_atoms, sizeof(int), FARGS);
     back_order = (int *)allocator(num_atoms, sizeof(int), FARGS);

     count = 0;
     if(resid[0] != NULL) {
          for(i = 0; i < num_atoms; i++) {
               if(atom_mass[i] == 0.0) {
                    atom_order[i] = -1;
                    back_order[i] = -1;
                    continue;
               }
               atom_order[i] = count;
               back_order[i] = count;
               count++;
          }                               /* end for(i = 0; i < num_atoms; i++) */

          true_atoms = count;
          return;
     }                                    /* end if(resid[0] != NULL) */

     j = 0;
     k = 0;

     while(j < num_atoms) {
          k++;
          for(i = 0; i < num_atoms; i++) {
               if(k == atom_segno[i]) {
                    if(atom_mass[i] == 0.0) {
                         atom_order[j] = -1;
                         back_order[i] = -1;
                         j++;
                         continue;
                    }
                    atom_order[j] = i;
                    back_order[i] = count;
                    j++;
                    count++;
               }                           /* end if(k == atom_segno[i]) */
          }                                /* end for(i = 0; i < num_atoms; i++) */
     }                                     /* while(j < num_atoms) */

     true_atoms = count;
     return;
}
